@@ -1,7 +1,6 @@
 // SPDX-License-Identifier: AGPL-3.0-or-later
-// SPDX-FileCopyrightText: 2022-2023 grommunio GmbH
+// SPDX-FileCopyrightText: 2022–2025 grommunio GmbH
 // This file is part of Gromox.
-
 #pragma once
 #include <cstdint>
 #include <list>
@@ -51,15 +50,14 @@ struct MessageInstanceKey {
 };
 
 using ExmdbSubscriptionKey = std::pair<std::string, uint32_t>;
-using SubscriptionKey = uint32_t;
-using ContextWakeupKey = int;
+using ContextKey = int;
 
 struct EmbeddedInstanceKey {
 	std::string dir;
 	uint32_t aid;
 
 	inline bool operator==(const EmbeddedInstanceKey& o) const
-	{return dir == o.dir && aid == o.aid;}
+	{ return aid == o.aid && dir == o.dir; }
 };
 
 } // namespace gromox::EWS::detail
@@ -93,10 +91,9 @@ class EWSPlugin {
 
 	EWSPlugin();
 	~EWSPlugin();
-	http_status proc(int, const void*, uint64_t);
-	static BOOL preproc(int);
-
-	bool logEnabled(const std::string_view&) const;
+	http_status proc(detail::ContextKey, const void*, uint64_t);
+	static BOOL preproc(detail::ContextKey);
+	bool logEnabled(const std::string_view) const;
 
 	struct _exmdb {
 		_exmdb();
@@ -135,7 +132,7 @@ class EWSPlugin {
 		std::mutex lock; ///< I/O mutex
 		std::vector<detail::ExmdbSubscriptionKey> inner_subs; ///< Exmdb subscription keys
 		std::list<Structures::sNotificationEvent> events; ///< Events that occured since last check
-		std::optional<int> waitingContext; ///< ID of context waiting for events
+		detail::ContextKey waitingContext = -1; ///< ID of context waiting for events
 	};
 
 	void event(const char*, BOOL, uint32_t, const DB_NOTIFY*) const;
@@ -149,7 +146,7 @@ class EWSPlugin {
 	detail::ExmdbSubscriptionKey subscribe(const std::string&, uint16_t, bool, uint64_t, detail::SubscriptionKey) const;
 	std::shared_ptr<SubManager> get_submgr(detail::SubscriptionKey, uint32_t) const;
 	std::string timestamp() const;
-	void unlinkSubscription(int) const;
+	void unlinkSubscription(detail::ContextKey) const;
 	bool unsubscribe(detail::SubscriptionKey, const char*) const;
 	void unsubscribe(const detail::ExmdbSubscriptionKey&) const;
 	void wakeContext(int, std::chrono::milliseconds) const;
@@ -170,8 +167,8 @@ class EWSPlugin {
 	std::chrono::milliseconds cache_message_instance_lifetime{30'000}; ///< Lifetime of message instances
 	std::chrono::milliseconds event_stream_interval{45'000}; ///< How often to send updates for GetStreamingEvents
 
-	int retr(int);
-	void term(int);
+	int retr(detail::ContextKey);
+	void term(detail::ContextKey);
 
 	private:
 	template<typename T> using sptr = std::shared_ptr<T>;
@@ -179,23 +176,79 @@ class EWSPlugin {
 	struct DebugCtx;
 
 	struct WakeupNotify {
-		inline explicit WakeupNotify(int id) : ID(id) {}
-		int ID;
+		inline explicit WakeupNotify(detail::ContextKey id) : ctx_id(id) {}
+		detail::ContextKey ctx_id;
 		~WakeupNotify();
 	};
 
-	using CacheKey = std::variant<detail::AttachmentInstanceKey, detail::MessageInstanceKey, detail::SubscriptionKey, detail::ContextWakeupKey, detail::EmbeddedInstanceKey>;
+	using CacheKey = std::variant<detail::AttachmentInstanceKey, detail::MessageInstanceKey, detail::SubscriptionKey, detail::ContextKey, detail::EmbeddedInstanceKey>;
 	using CacheObj = std::variant<sptr<ExmdbInstance>, sptr<SubManager>, sptr<WakeupNotify>>;
 
 	static const std::unordered_map<std::string, Handler> requestMap;
 
-	static http_status fault(int, http_status, const std::string_view&);
+	static http_status fault(detail::ContextKey, http_status, const std::string_view);
 
 	mutable std::mutex subscriptionLock;
 	mutable std::unordered_map<detail::ExmdbSubscriptionKey, detail::SubscriptionKey> subscriptions;
 
 	std::vector<std::unique_ptr<EWSContext>> contexts;
-	mutable ObjectCache<CacheKey, CacheObj> cache;
+
+	/**
+	 * @brief      Timed object cache
+	 *
+	 * Objects are stored for a limited time and automatically removed once they
+	 * expire.
+	 *
+	 * Once stored, only copies of the stored elements can be retrieved to avoid
+	 * asynchronous deconstruction during access (use std::shared_ptr).
+	 */
+	class ObjectCache {
+		private:
+		mutable std::mutex objectLock; ///< Mutex to protect object map
+		using Container = std::pair<gromox::time_point, CacheObj>;
+		std::unordered_map<CacheKey, Container> objects; ///< Stored objects
+		using node_t = typename decltype(objects)::node_type;
+
+		std::condition_variable notify; ///< CV to signal stopping
+		std::thread scanThread; ///< Thread used for periodic scanning
+		gromox::atomic_bool running{false}; ///< Whether the scanner is running
+
+		public:
+		/**
+		 * @brief      Clear cache and stop scanner
+		 */
+		~ObjectCache() { stop(); }
+		void run(std::chrono::milliseconds);
+		void stop();
+
+		/**
+		 * @brief      Add object to cache
+		 *
+		 * @param      lifespan  Object life span
+		 * @param      key       Object key
+		 * @param      args      Object constructor arguments
+		 *
+		 * @tparam     KeyArg    Type to derive key from
+		 * @tparam     Args      Object constructor arguments
+		 *
+		 * @return     true if emplaced, false if already present
+		 */
+		template<typename K, typename... Args>
+		bool emplace(std::chrono::milliseconds lifespan, K &&key, Args &&...args)
+		{
+			std::lock_guard guard(objectLock);
+			auto res = objects.try_emplace(CacheKey(key), tp_now() + lifespan, std::forward<Args...>(args)...);
+			return res.second;
+		}
+
+		CacheObj get(const CacheKey &) const;
+		CacheObj get(const CacheKey &, std::chrono::milliseconds);
+		void evict(const CacheKey &);
+		void scan();
+		void periodicScan(std::chrono::milliseconds);
+	};
+
+	mutable ObjectCache cache;
 
 	std::unique_ptr<DebugCtx> debug;
 	std::vector<std::string> logFilters;
@@ -203,7 +256,7 @@ class EWSPlugin {
 	bool invertFilter = true;
 	bool teardown = false;
 
-	http_status dispatch(int, HTTP_AUTH_INFO&, const void*, uint64_t);
+	http_status dispatch(detail::ContextKey, HTTP_AUTH_INFO &, const void *, uint64_t);
 	void loadConfig();
 };
 
@@ -216,11 +269,9 @@ class EWSContext {
 
 	enum State : uint8_t {S_DEFAULT, S_WRITE, S_DONE, S_STREAM_NOTIFY};
 
-	EWSContext(int, HTTP_AUTH_INFO, const char *, uint64_t, EWSPlugin &);
+	EWSContext(detail::ContextKey, const HTTP_AUTH_INFO &, const char *, uint64_t, EWSPlugin &);
 	~EWSContext();
-
-	EWSContext(const EWSContext&) = delete;
-	EWSContext(EWSContext&&) = delete;
+	NOMOVE(EWSContext);
 
 	Structures::sFolder create(const std::string&, const Structures::sFolderSpec&, const Structures::sFolder&) const;
 	Structures::sItem create(const std::string&, const Structures::sFolderSpec&, const MESSAGE_CONTENT&) const;
@@ -229,22 +280,23 @@ class EWSContext {
 	const char* effectiveUser(const Structures::sFolderSpec&) const;
 	void enableEventStream(int);
 	std::string essdn_to_username(const std::string&) const;
+	std::string exportContent(const std::string&, const MESSAGE_CONTENT&, const std::string&) const;
 	std::string get_maildir(const Structures::tMailbox&) const;
 	std::string get_maildir(const std::string&) const;
 	uint32_t getAccountId(const std::string&, bool) const;
 	std::string getDir(const Structures::sFolderSpec&) const;
 	TAGGED_PROPVAL getFolderEntryId(const std::string&, uint64_t) const;
-	template<typename T> const T* getFolderProp(const std::string&, uint64_t, uint32_t) const;
-	TPROPVAL_ARRAY getFolderProps(const std::string&, uint64_t, const PROPTAG_ARRAY&) const;
+	template<typename T> const T *getFolderProp(const std::string &, uint64_t, proptag_t) const;
+	TPROPVAL_ARRAY getFolderProps(const std::string&, uint64_t, proptag_cspan) const;
 	std::pair<std::list<Structures::sNotificationEvent>, bool> getEvents(const Structures::tSubscriptionId&) const;
 	TAGGED_PROPVAL getFolderEntryId(const Structures::sFolderSpec&) const;
 	TPROPVAL_ARRAY getFolderProps(const Structures::sFolderSpec&, const PROPTAG_ARRAY&) const;
 	TAGGED_PROPVAL getItemEntryId(const std::string&, uint64_t) const;
-	template<typename T> const T* getItemProp(const std::string&, uint64_t, uint32_t) const;
+	template<typename T> const T *getItemProp(const std::string &, uint64_t, proptag_t) const;
 	TPROPVAL_ARRAY getItemProps(const std::string&, uint64_t, const PROPTAG_ARRAY&) const;
 	GUID getMailboxGuid(const std::string&) const;
 	Structures::sMailboxInfo getMailboxInfo(const std::string&, bool) const;
-	uint16_t getNamedPropId(const std::string&, const PROPERTY_NAME&, bool=false) const;
+	propid_t getNamedPropId(const std::string &, const PROPERTY_NAME &, bool = false) const;
 	PROPID_ARRAY getNamedPropIds(const std::string&, const PROPNAME_ARRAY&, bool=false) const;
 	void getNamedTags(const std::string&, Structures::sShape&, bool=false) const;
 	Structures::sAttachment loadAttachment(const std::string&,const Structures::sAttachmentId&) const;
@@ -273,6 +325,7 @@ class EWSContext {
 	void send(const std::string &dir, uint64_t log_msg_id, const MESSAGE_CONTENT &) const;
 	BINARY serialize(const XID&) const;
 	bool streamEvents(const Structures::tSubscriptionId&) const;
+	void rcpt_add_unique(TARRAY_SET *, gromox::EWS::Structures::tEmailAddressType, uint32_t) const;
 	MCONT_PTR toContent(const std::string&, std::string&) const;
 	MCONT_PTR toContent(const std::string&, const Structures::sFolderSpec&, Structures::sItem&, bool) const;
 	Structures::tSubscriptionId subscribe(const Structures::tPullSubscriptionRequest&) const;
@@ -287,7 +340,7 @@ class EWSContext {
 	gromox::time_duration age() const { return tp_now() - m_created; }
 	void experimental(const char*) const;
 
-	inline int ID() const {return m_ID;}
+	inline detail::ContextKey context_id() const { return m_ctx_id; }
 	inline const HTTP_AUTH_INFO& auth_info() const {return m_auth_info;}
 	inline const EWSPlugin& plugin() const {return m_plugin;}
 	inline const SOAP::Envelope& request() const {return m_request;}
@@ -303,7 +356,7 @@ class EWSContext {
 	static void* alloc(size_t);
 	template<typename T> static T* alloc(size_t=1);
 	template<typename T, typename... Args> static T* construct(Args&&...);
-	static char* cpystr(const std::string_view&);
+	static char *cpystr(const std::string_view);
 
 	static void assertIdType(Structures::tBaseItemId::IdType, Structures::tBaseItemId::IdType);
 	static void ext_error(pack_result, const char* = nullptr, const char* = nullptr);
@@ -344,9 +397,9 @@ private:
 	inline void updateProps(Structures::tItem&, Structures::sShape&, const TPROPVAL_ARRAY&) const {}
 	void updateProps(Structures::tCalendarItem&, Structures::sShape&, const TPROPVAL_ARRAY&) const;
 
-	PROPERTY_NAME* getPropertyName(const std::string&, uint16_t) const;
+	PROPERTY_NAME *getPropertyName(const std::string &, propid_t) const;
 
-	int m_ID = 0;
+	detail::ContextKey m_ctx_id = -1;
 	http_status m_code = http_status::ok;
 	State m_state = S_DEFAULT;
 	bool m_log = false;
@@ -372,8 +425,8 @@ private:
  *
  * @return     Pointer to property or nullptr if not found.
  */
-template<typename T>
-const T* EWSContext::getFolderProp(const std::string& dir, uint64_t mid, uint32_t tag) const
+template<typename T> const T *
+EWSContext::getFolderProp(const std::string &dir, uint64_t mid, proptag_t tag) const
 {
 	return static_cast<const T *>(getFolderProp(dir, mid, tag));
 }
@@ -389,8 +442,8 @@ const T* EWSContext::getFolderProp(const std::string& dir, uint64_t mid, uint32_
  *
  * @return     Pointer to property or nullptr if not found.
  */
-template<typename T>
-const T* EWSContext::getItemProp(const std::string& dir, uint64_t mid, uint32_t tag) const
+template<typename T> const T *
+EWSContext::getItemProp(const std::string &dir, uint64_t mid, proptag_t tag) const
 {
 	return static_cast<const T*>(getItemProp(dir, mid, tag));
 }
