@@ -176,7 +176,7 @@ static int cu_test_delegate_perm_MD(const char *account,
 			return 1;
 	return 0;
 } catch (const std::bad_alloc &) {
-	mlog(LV_ERR, "E-2056: ENOMEM");
+	mlog(LV_ERR, "%s: ENOMEM", __func__);
 	return false;
 }
 
@@ -358,7 +358,7 @@ BOOL common_util_build_environment() try
 	g_env_key = std::make_unique<env_context>();
 	return TRUE;
 } catch (const std::bad_alloc &) {
-	mlog(LV_ERR, "E-1977: ENOMEM");
+	mlog(LV_ERR, "%s: ENOMEM", __func__);
 	return false;
 }
 
@@ -705,7 +705,7 @@ std::string cu_fid_to_entryid_s(const store_object &store, uint64_t folder_id) t
 	out.resize(ep.m_offset);
 	return out;
 } catch (const std::bad_alloc &) {
-	mlog(LV_ERR, "E-2257: ENOMEM");
+	mlog(LV_ERR, "%s: ENOMEM", __func__);
 	return {};
 }
 
@@ -754,7 +754,7 @@ std::string cu_fid_to_sk_s(const store_object &store, uint64_t folder_id) try
 	out.resize(ep.m_offset);
 	return out;
 } catch (const std::bad_alloc &) {
-	mlog(LV_ERR, "E-2258: ENOMEM");
+	mlog(LV_ERR, "%s: ENOMEM", __func__);
 	return {};
 }
 
@@ -829,7 +829,7 @@ std::string cu_mid_to_entryid_s(const store_object &store, uint64_t folder_id,
 	out.resize(ep.m_offset);
 	return out;
 } catch (const std::bad_alloc &) {
-	mlog(LV_ERR, "E-2259: ENOMEM");
+	mlog(LV_ERR, "%s: ENOMEM", __func__);
 	return {};
 }
 
@@ -911,7 +911,7 @@ std::string cu_mid_to_sk_s(const store_object &store, uint64_t msg_id) try
 	out.resize(ep.m_offset);
 	return out;
 } catch (const std::bad_alloc &) {
-	mlog(LV_ERR, "E-2260: ENOMEM");
+	mlog(LV_ERR, "%s: ENOMEM", __func__);
 	return {};
 }
 
@@ -942,7 +942,7 @@ std::string cu_xid_to_bin_s(const XID &xid) try
 	out.resize(ep.m_offset);
 	return out;
 } catch (const std::bad_alloc &) {
-	mlog(LV_ERR, "E-2261: ENOMEM");
+	mlog(LV_ERR, "%s: ENOMEM", __func__);
 	return {};
 }
 
@@ -1057,7 +1057,7 @@ static BOOL common_util_get_propname(propid_t propid, PROPERTY_NAME **pppropname
 	*pppropname = propnames.ppropname;
 	return TRUE;
 } catch (const std::bad_alloc &) {
-	mlog(LV_ERR, "E-2230: ENOMEM");
+	mlog(LV_ERR, "%s: ENOMEM", __func__);
 	return false;
 }
 
@@ -1066,9 +1066,7 @@ ec_error_t cu_send_message(store_object *pstore, message_object *msg,
 {
 	uint64_t message_id = msg->get_id();
 	void *pvalue;
-	BOOL b_result;
-	BOOL b_private;
-	BOOL b_partial;
+	BOOL b_private, b_partial = false;
 	int account_id;
 	uint64_t new_id;
 	uint64_t folder_id;
@@ -1162,15 +1160,14 @@ ec_error_t cu_send_message(store_object *pstore, message_object *msg,
 		if (!exmdb_client->clear_submit(pstore->get_dir(), message_id, false))
 			return ecWarnWithErrors;
 		if (!exmdb_client->movecopy_message(pstore->get_dir(), cpid,
-		    message_id, folder_id, new_id, TRUE, &b_result))
+		    message_id, folder_id, new_id, TRUE, &b_partial) || b_partial)
 			return ecWarnWithErrors;
 		return ecSuccess;
 	} else if (b_delete) {
 		const EID_ARRAY ids = {1, &message_id};
 		if (!exmdb_client->delete_messages(pstore->get_dir(), cpid,
-		    nullptr, parent_id, &ids, TRUE, &b_result))
+		    nullptr, parent_id, &ids, TRUE, &b_partial) || b_partial)
 			/* ignore */;
-		b_result = !b_partial ? TRUE : false;
 		return ecSuccess;
 	}
 	if (!exmdb_client->clear_submit(pstore->get_dir(), message_id, false))
@@ -1182,11 +1179,11 @@ ec_error_t cu_send_message(store_object *pstore, message_object *msg,
 
 	const EID_ARRAY ids = {1, &message_id};
 	if (!exmdb_client->movecopy_messages(pstore->get_dir(), cpid, false,
-	    STORE_OWNER_GRANTED, parent_id, folder_id, false, &ids, &b_partial))
+	    STORE_OWNER_GRANTED, parent_id, folder_id, false, &ids, &b_partial) || b_partial)
 		return ecWarnWithErrors;
 	return ecSuccess;
 } catch (const std::bad_alloc &) {
-	mlog(LV_ERR, "E-2549: ENOMEM");
+	mlog(LV_ERR, "%s: ENOMEM", __func__);
 	return ecServerOOM;
 }
 
@@ -1209,7 +1206,7 @@ void common_util_notify_receipt(const char *username, int type,
 	if (ret != ecSuccess)
 		mlog(LV_ERR, "E-1193: cu_send_mail: %xh", static_cast<unsigned int>(ret));
 } catch (const std::bad_alloc &) {
-	mlog(LV_ERR, "E-2038: ENOMEM");
+	mlog(LV_ERR, "%s: ENOMEM", __func__);
 }
 
 static MOVECOPY_ACTION *cu_cvt_from_zmovecopy(const ZMOVECOPY_ACTION &src)
@@ -1381,7 +1378,7 @@ std::string cu_to_store_entryid_s(const store_object &store) try
 	out.resize(ep.m_offset);
 	return out;
 } catch (const std::bad_alloc &) {
-	mlog(LV_ERR, "E-2262: ENOMEM");
+	mlog(LV_ERR, "%s: ENOMEM", __func__);
 	return {};
 }
 
@@ -1462,7 +1459,7 @@ ec_error_t cu_remote_copy_message(store_object *src_store, uint64_t message_id,
 		return ecError;
 	if (pmsgctnt == nullptr)
 		return ecSuccess;
-	static constexpr uint32_t tags[] = {
+	static constexpr proptag_t tags[] = {
 		PR_CONVERSATION_ID, PR_DISPLAY_TO,
 		PR_DISPLAY_TO_A, PR_DISPLAY_CC,
 		PR_DISPLAY_CC_A, PR_DISPLAY_BCC, PR_DISPLAY_BCC_A, PidTagMid,
@@ -1511,7 +1508,7 @@ static ec_error_t cu_create_folder(store_object *pstore, uint64_t parent_id,
 	PERMISSION_DATA permission_row;
 	TAGGED_PROPVAL propval_buff[10];
 
-	static constexpr uint32_t tags[] = {	
+	static constexpr proptag_t tags[] = {	
 		PR_ACCESS, PR_ACCESS_LEVEL, PR_ADDRESS_BOOK_ENTRYID,
 		PR_ASSOC_CONTENT_COUNT, PR_ATTR_READONLY,
 		PR_CONTENT_COUNT, PR_CONTENT_UNREAD,
@@ -1739,7 +1736,7 @@ BOOL common_util_message_to_rfc822(store_object *pstore, uint64_t inst_id,
 	}
 	return TRUE;
 } catch (const std::bad_alloc &) {
-	mlog(LV_ERR, "E-2548: ENOMEM");
+	mlog(LV_ERR, "%s: ENOMEM", __func__);
 	return false;
 }
 
@@ -1752,9 +1749,13 @@ static void zc_unwrap_clearsigned(MAIL &ma) try
 	part = part->get_child();
 	if (part == nullptr)
 		return;
-	ma.load_from_str(part->head_begin, part->content_begin + part->content_length - part->head_begin);
+	/*
+	 * head_begin is a pointer into the caller's peml_bin block,
+	 * so lifetimes should still be ok.
+	 */
+	ma.refonly_parse(part->head_begin, part->content_begin + part->content_length - part->head_begin);
 } catch (const std::bad_alloc &) {
-	mlog(LV_ERR, "E-1996: ENOMEM");
+	mlog(LV_ERR, "%s: ENOMEM", __func__);
 }
 
 MESSAGE_CONTENT *cu_rfc822_to_message(store_object *pstore,
@@ -1763,7 +1764,7 @@ MESSAGE_CONTENT *cu_rfc822_to_message(store_object *pstore,
 	char charset[32];
 	auto pinfo = zs_get_info();
 	MAIL imail;
-	if (!imail.load_from_str(peml_bin->pc, peml_bin->cb))
+	if (!imail.refonly_parse(peml_bin->pc, peml_bin->cb))
 		return NULL;
 	if (mxf_flags & MXF_UNWRAP_SMIME_CLEARSIGNED)
 		zc_unwrap_clearsigned(imail);
@@ -1812,7 +1813,7 @@ BOOL common_util_message_to_ical(store_object *pstore, uint64_t message_id,
 	pical_bin->pc = common_util_dup(tmp_buff);
 	return pical_bin->pc != nullptr ? TRUE : FALSE;
 } catch (const std::bad_alloc &) {
-	mlog(LV_ERR, "E-2183: ENOMEM");
+	mlog(LV_ERR, "%s: ENOMEM", __func__);
 	return false;
 }
 
@@ -1837,7 +1838,7 @@ message_ptr cu_ical_to_message(store_object *pstore, const BINARY *pical_bin) tr
 	return oxcical_import_single(tmzone, ical, common_util_alloc,
 	       common_util_get_propids_create, common_util_username_to_entryid);
 } catch (const std::bad_alloc &) {
-	mlog(LV_ERR, "E-2184: ENOMEM");
+	mlog(LV_ERR, "%s: ENOMEM", __func__);
 	return nullptr;
 }
 
@@ -1860,7 +1861,7 @@ ec_error_t cu_ical_to_message2(store_object *store, char *ical_data,
 	       common_util_get_propids_create,
 	       common_util_username_to_entryid, msgvec);
 } catch (const std::bad_alloc &) {
-	mlog(LV_ERR, "E-2185: ENOMEM");
+	mlog(LV_ERR, "%s: ENOMEM", __func__);
 	return ecServerOOM;
 }
 
@@ -1926,7 +1927,7 @@ ec_error_t cu_vcf_to_message2(store_object *store, char *vcf_data,
 	}
 	return ecSuccess;
 } catch (const std::bad_alloc &) {
-	mlog(LV_ERR, "E-2048: ENOMEM");
+	mlog(LV_ERR, "%s: ENOMEM", __func__);
 	return ecServerOOM;
 }
 
@@ -2007,7 +2008,7 @@ ec_error_t cu_fbdata_to_ical(const char *user, const char *fbuser,
 		return ecServerOOM;
 	return ecSuccess;
 } catch (const std::bad_alloc &) {
-	mlog(LV_ERR, "E-2188: ENOMEM");
+	mlog(LV_ERR, "%s: ENOMEM", __func__);
 	return ecServerOOM;
 }
 
