@@ -180,7 +180,7 @@ BOOL container_object::load_user_table(const RESTRICTION *prestriction) try
 	BINARY *pparent_entryid = nullptr;
 	LONG_ARRAY *pminid_array;
 	proptag_t proptag_buff[25];
-	static constexpr uint32_t tmp_proptags[] = {
+	static constexpr proptag_t tmp_proptags[] = {
 		PR_NICKNAME, PR_SURNAME, PR_GIVEN_NAME, PR_MIDDLE_NAME,
 		PR_TITLE, PR_PRIMARY_TELEPHONE_NUMBER,
 		PR_MOBILE_TELEPHONE_NUMBER, PR_HOME_ADDRESS_STREET, PR_COMMENT,
@@ -234,22 +234,17 @@ BOOL container_object::load_user_table(const RESTRICTION *prestriction) try
 		proptags.pproptag = proptag_buff;
 		if (!container_object_get_pidlids(&proptags))
 			return FALSE;
-		proptags.pproptag[proptags.count++] = PR_DISPLAY_NAME;
-		proptags.pproptag[proptags.count++] = PR_NICKNAME;
-		proptags.pproptag[proptags.count++] = PR_TITLE;
-		proptags.pproptag[proptags.count++] = PR_SURNAME;
-		proptags.pproptag[proptags.count++] = PR_GIVEN_NAME;
-		proptags.pproptag[proptags.count++] = PR_MIDDLE_NAME;
-		proptags.pproptag[proptags.count++] = PR_PRIMARY_TELEPHONE_NUMBER;
-		proptags.pproptag[proptags.count++] = PR_MOBILE_TELEPHONE_NUMBER;
-		proptags.pproptag[proptags.count++] = PR_HOME_ADDRESS_STREET;
-		proptags.pproptag[proptags.count++] = PR_COMMENT;
-		proptags.pproptag[proptags.count++] = PR_COMPANY_NAME;
-		proptags.pproptag[proptags.count++] = PR_DEPARTMENT_NAME;
-		proptags.pproptag[proptags.count++] = PR_OFFICE_LOCATION;
-		proptags.pproptag[proptags.count++] = PR_CREATION_TIME;
-		proptags.pproptag[proptags.count++] = PR_MESSAGE_CLASS;
-		proptags.pproptag[proptags.count++] = PidTagMid;
+		static constexpr proptag_t ntags[] = {
+			PR_DISPLAY_NAME, PR_NICKNAME, PR_TITLE, PR_SURNAME,
+			PR_GIVEN_NAME, PR_MIDDLE_NAME,
+			PR_PRIMARY_TELEPHONE_NUMBER,
+			PR_MOBILE_TELEPHONE_NUMBER, PR_HOME_ADDRESS_STREET,
+			PR_COMMENT, PR_COMPANY_NAME, PR_DEPARTMENT_NAME,
+			PR_OFFICE_LOCATION, PR_CREATION_TIME, PR_MESSAGE_CLASS,
+			PidTagMid,
+		};
+		for (auto t : ntags)
+			proptags.emplace_back(t);
 		if (!exmdb_client->query_table(pinfo->get_maildir(), nullptr,
 		    pinfo->cpid, table_id, &proptags, 0, row_num, &tmp_set))
 			return FALSE;
@@ -389,7 +384,7 @@ BOOL container_object_fetch_special_property(uint8_t special_type,
 		EMSAB_ENTRYID_view ab_entryid;
 		ab_entryid.flags = 0;
 		ab_entryid.type = DT_CONTAINER;
-		ab_entryid.px500dn = special_type == SPECIAL_CONTAINER_GAL ? "/" : "/exmdb";
+		ab_entryid.px500dn = special_type == ab_tree::minid::SC_GAL ? "/" : "/exmdb";
 		bv->pv = common_util_alloc(128);
 		if (bv->pv == nullptr ||
 		    !ext_push.init(static_cast<BINARY *>(pvalue)->pb, 128, 0) ||
@@ -414,7 +409,7 @@ BOOL container_object_fetch_special_property(uint8_t special_type,
 		*ppvalue = pvalue;
 		return TRUE;
 	case PR_DISPLAY_NAME:
-		*ppvalue = special_type == SPECIAL_CONTAINER_GAL ?
+		*ppvalue = special_type == ab_tree::minid::SC_GAL ?
 		           deconst("Global Address List") :
 		           deconst("Gromox Contact Folders");
 		return TRUE;
@@ -488,7 +483,7 @@ static BOOL container_object_fetch_folder_properties(
 				         MAPI_ABCONT, UINT32_MAX);
 			} else if (folder_id == rop_util_make_eid_ex(
 			    1, PRIVATE_FID_CONTACTS)) {
-				if (!container_object_fetch_special_property(SPECIAL_CONTAINER_PROVIDER,
+				if (!container_object_fetch_special_property(ab_tree::minid::SC_PROVIDER,
 				    PR_ENTRYID, &pvalue))
 					return FALSE;
 			} else {
@@ -555,7 +550,7 @@ static BOOL container_object_fetch_folder_properties(
 
 static const PROPTAG_ARRAY* container_object_get_folder_proptags()
 {
-	static constexpr uint32_t p[] = {
+	static constexpr proptag_t p[] = {
 		PidTagFolderId, PR_SUBFOLDERS, PR_DISPLAY_NAME,
 		PR_CONTAINER_CLASS, PR_FOLDER_PATHNAME,
 		PidTagParentFolderId, PR_ATTR_HIDDEN,
@@ -579,9 +574,9 @@ BOOL container_object::get_properties(const PROPTAG_ARRAY *pproptags,
 		return container_object_fetch_folder_properties(
 					&tmp_propvals, pproptags, ppropvals);
 	}
-	if (pcontainer->id.abtree_id.minid == SPECIAL_CONTAINER_EMPTY)
-		return container_object_fetch_special_properties(
-			SPECIAL_CONTAINER_PROVIDER, pproptags, ppropvals);
+	if (pcontainer->id.abtree_id.minid == ab_tree::minid::SC_EMPTY)
+		return container_object_fetch_special_properties(ab_tree::minid::SC_PROVIDER,
+		       pproptags, ppropvals);
 	auto pbase = ab_tree::AB.get(pcontainer->id.abtree_id.base_id);
 	if (!pbase)
 		return FALSE;
@@ -611,7 +606,7 @@ BOOL container_object::get_container_table_num(BOOL b_depth, uint32_t *pnum)
 void container_object_get_container_table_all_proptags(
 	PROPTAG_ARRAY *pproptags)
 {
-	static constexpr uint32_t p[] = {
+	static constexpr proptag_t p[] = {
 		PR_ENTRYID, PR_CONTAINER_FLAGS, PR_DEPTH, PR_INSTANCE_KEY,
 		PR_EMS_AB_CONTAINERID, PR_DISPLAY_NAME, PR_EMS_AB_IS_MASTER,
 		PR_EMS_AB_PARENT_ENTRYID, PR_AB_PROVIDER_ID,
@@ -620,9 +615,9 @@ void container_object_get_container_table_all_proptags(
 	pproptags->pproptag = deconst(p);
 }
 
-static BOOL
+static bool
 container_object_get_specialtables_from_node(const ab_tree::ab_node& node,
-    const PROPTAG_ARRAY *pproptags, BOOL b_depth, TARRAY_SET *pset)
+    const PROPTAG_ARRAY *pproptags, TARRAY_SET *pset)
 {
 	TPROPVAL_ARRAY **pparray;
 	auto count = strange_roundup(pset->count, SR_GROW_TPROPVAL_ARRAY);
@@ -646,9 +641,8 @@ container_object_get_specialtables_from_node(const ab_tree::ab_node& node,
 	return TRUE;
 }
 
-static BOOL container_object_query_folder_hierarchy(
-	uint64_t folder_id, const PROPTAG_ARRAY *pproptags,
-	BOOL b_depth, TARRAY_SET *pset)
+static bool container_object_query_folder_hierarchy(uint64_t folder_id,
+    const PROPTAG_ARRAY *pproptags, TARRAY_SET *pset)
 {
 	uint32_t row_num;
 	uint32_t table_id;
@@ -713,24 +707,24 @@ BOOL container_object::query_container_table(const PROPTAG_ARRAY *pproptags,
 		return FALSE;
 	if (CONTAINER_TYPE_FOLDER == pcontainer->type) {
 		if (!container_object_query_folder_hierarchy(pcontainer->id.exmdb_id.folder_id,
-		    pproptags, b_depth, &tmp_set))
+		    pproptags, &tmp_set))
 			return FALSE;	
 	} else {
 		auto pbase = ab_tree::AB.get(pcontainer->id.abtree_id.base_id);
 		if (!pbase)
 			return FALSE;
-		if (pcontainer->id.abtree_id.minid == SPECIAL_CONTAINER_ROOT) {
+		if (pcontainer->id.abtree_id.minid == ab_tree::minid::SC_ROOT) {
 			tmp_set.pparray[tmp_set.count] = cu_alloc<TPROPVAL_ARRAY>();
 			if (tmp_set.pparray[tmp_set.count] == nullptr)
 				return FALSE;
-			if (!container_object_fetch_special_properties(SPECIAL_CONTAINER_GAL,
+			if (!container_object_fetch_special_properties(ab_tree::minid::SC_GAL,
 			    pproptags, tmp_set.pparray[tmp_set.count]))
 				return FALSE;
 			tmp_set.count ++;
 			tmp_set.pparray[tmp_set.count] = cu_alloc<TPROPVAL_ARRAY>();
 			if (tmp_set.pparray[tmp_set.count] == nullptr)
 				return FALSE;
-			if (!container_object_fetch_special_properties(SPECIAL_CONTAINER_PROVIDER,
+			if (!container_object_fetch_special_properties(ab_tree::minid::SC_PROVIDER,
 			    pproptags, tmp_set.pparray[tmp_set.count]))
 				return FALSE;
 			tmp_set.count ++;
@@ -749,13 +743,13 @@ BOOL container_object::query_container_table(const PROPTAG_ARRAY *pproptags,
 			tmp_set.count ++;
 			if (b_depth && !container_object_query_folder_hierarchy(
 			    rop_util_make_eid_ex(1, PRIVATE_FID_CONTACTS),
-			    pproptags, TRUE, &tmp_set))
+			    pproptags, &tmp_set))
 				return FALSE;
 			for (auto it = pbase->dbegin(); it != pbase->dend(); ++it)
 				if (!container_object_get_specialtables_from_node(it,
-				    pproptags, b_depth, &tmp_set))
+				    pproptags, &tmp_set))
 					return FALSE;
-		} else if (pcontainer->id.abtree_id.minid == SPECIAL_CONTAINER_GAL) {
+		} else if (pcontainer->id.abtree_id.minid == ab_tree::minid::SC_GAL) {
 			/* no subordinates */
 		} else {
 			if (!pbase->exists(pcontainer->id.abtree_id.minid)) {
@@ -823,7 +817,7 @@ BOOL container_object::get_user_table_num(uint32_t *pnum)
 void container_object_get_user_table_all_proptags(
 	PROPTAG_ARRAY *pproptags)
 {
-	static constexpr uint32_t p[] = {
+	static constexpr proptag_t p[] = {
 		PR_DISPLAY_NAME, PR_NICKNAME, PR_SURNAME, PR_GIVEN_NAME,
 		PR_MIDDLE_NAME, PR_TITLE, PR_PRIMARY_TELEPHONE_NUMBER,
 		PR_MOBILE_TELEPHONE_NUMBER, PR_HOME_ADDRESS_STREET, PR_COMMENT,
@@ -886,8 +880,14 @@ BOOL container_object::query_user_table(const PROPTAG_ARRAY *pproptags,
 			for (size_t i = first_pos; i < first_pos+row_count &&
 			     i < pcontainer->contents.pminid_array->count; ++i) {
 				ab_tree::ab_node node(pbase, pcontainer->contents.pminid_array->pl[i]);
-				if (!node.exists() || node.hidden() & AB_HIDE_FROM_AL)
+				if (!node.exists())
 					continue;
+				/*
+				 * No testing for HIDE* here, because
+				 * pminid_array is already prefiltered (when it
+				 * was made; ab_tree_match_minids is called by
+				 * load_user_table).
+				 */
 				pset->pparray[pset->count] = cu_alloc<TPROPVAL_ARRAY>();
 				if (pset->pparray[pset->count] == nullptr)
 					return FALSE;
@@ -896,7 +896,7 @@ BOOL container_object::query_user_table(const PROPTAG_ARRAY *pproptags,
 					return FALSE;	
 				pset->count ++;
 			}
-		} else if (pcontainer->id.abtree_id.minid == SPECIAL_CONTAINER_GAL) {
+		} else if (pcontainer->id.abtree_id.minid == ab_tree::minid::SC_GAL) {
 			for (auto it = pbase->ubegin() + first_pos; it != pbase->uend(); ++it) {
 				ab_tree::ab_node node(it);
 				if (node.hidden() & AB_HIDE_FROM_GAL)
@@ -911,7 +911,7 @@ BOOL container_object::query_user_table(const PROPTAG_ARRAY *pproptags,
 				if (pset->count == row_count)
 					break;
 			}
-		} else if (pcontainer->id.abtree_id.minid == SPECIAL_CONTAINER_EMPTY) {
+		} else if (pcontainer->id.abtree_id.minid == ab_tree::minid::SC_EMPTY) {
 			return TRUE;
 		} else {
 			ab_tree::ab_node node(pbase, pcontainer->id.abtree_id.minid);
