@@ -19,7 +19,7 @@ struct GX_EXPORT MAIL {
 	MAIL &operator=(MAIL &&);
 
 	void clear();
-	bool load_from_str(const char *in_buff, size_t length);
+	bool refonly_parse(const char *in_buff, size_t length);
 	bool serialize(STREAM *) const;
 	gromox::errno_t to_fd(int) const;
 	gromox::errno_t to_str(std::string &) const;
@@ -35,5 +35,5 @@ struct GX_EXPORT MAIL {
 	bool set_header(const char *hdr, const char *val);
 
 	SIMPLE_TREE tree{};
-	char *buffer = nullptr;
+	std::unique_ptr<char[]> buffer;
 };
