@@ -24,7 +24,8 @@ BOOL exmdb_client_remote::get_all_named_propids(const char *dir, PROPID_ARRAY *p
 	q.dir = deconst(dir);
 	if (!exmdb_client_do_rpc(&q, &r))
 		return false;
-	*propids = std::move(r.propids);
+	if (propids != nullptr)
+		*propids = std::move(r.propids);
 	return TRUE;
 }
 
@@ -39,7 +40,8 @@ BOOL exmdb_client_remote::get_named_propids(const char *dir, BOOL b_create, cons
 	q.ppropnames = deconst(ppropnames);
 	if (!exmdb_client_do_rpc(&q, &r))
 		return false;
-	*propids = std::move(r.propids);
+	if (propids != nullptr)
+		*propids = std::move(r.propids);
 	return TRUE;
 }
 
@@ -53,7 +55,8 @@ BOOL exmdb_client_remote::get_named_propnames(const char *dir, const PROPID_ARRA
 	q.ppropids = ppropids;
 	if (!exmdb_client_do_rpc(&q, &r))
 		return false;
-	*propnames = std::move(r.propnames);
+	if (propnames != nullptr)
+		*propnames = std::move(r.propnames);
 	return TRUE;
 }
 
@@ -67,8 +70,10 @@ BOOL exmdb_client_remote::get_mapping_guid(const char *dir, uint16_t replid, BOO
 	q.replid = replid;
 	if (!exmdb_client_do_rpc(&q, &r))
 		return false;
-	*b_found = std::move(r.b_found);
-	*guid = std::move(r.guid);
+	if (b_found != nullptr)
+		*b_found = std::move(r.b_found);
+	if (guid != nullptr)
+		*guid = std::move(r.guid);
 	return TRUE;
 }
 
@@ -82,8 +87,10 @@ BOOL exmdb_client_remote::get_mapping_replid(const char *dir, GUID guid, uint16_
 	q.guid = guid;
 	if (!exmdb_client_do_rpc(&q, &r))
 		return false;
-	*replid = std::move(r.replid);
-	*e_result = std::move(r.e_result);
+	if (replid != nullptr)
+		*replid = std::move(r.replid);
+	if (e_result != nullptr)
+		*e_result = std::move(r.e_result);
 	return TRUE;
 }
 
@@ -96,7 +103,8 @@ BOOL exmdb_client_remote::get_store_all_proptags(const char *dir, PROPTAG_ARRAY
 	q.dir = deconst(dir);
 	if (!exmdb_client_do_rpc(&q, &r))
 		return false;
-	*proptags = std::move(r.proptags);
+	if (proptags != nullptr)
+		*proptags = std::move(r.proptags);
 	return TRUE;
 }
 
@@ -111,7 +119,8 @@ BOOL exmdb_client_remote::get_store_properties(const char *dir, cpid_t cpid, con
 	q.pproptags = deconst(pproptags);
 	if (!exmdb_client_do_rpc(&q, &r))
 		return false;
-	*propvals = std::move(r.propvals);
+	if (propvals != nullptr)
+		*propvals = std::move(r.propvals);
 	return TRUE;
 }
 
@@ -126,7 +135,8 @@ BOOL exmdb_client_remote::set_store_properties(const char *dir, cpid_t cpid, con
 	q.ppropvals = deconst(ppropvals);
 	if (!exmdb_client_do_rpc(&q, &r))
 		return false;
-	*problems = std::move(r.problems);
+	if (problems != nullptr)
+		*problems = std::move(r.problems);
 	return TRUE;
 }
 
@@ -153,7 +163,8 @@ BOOL exmdb_client_remote::get_mbox_perm(const char *dir, const char *username, u
 	q.username = deconst(username);
 	if (!exmdb_client_do_rpc(&q, &r))
 		return false;
-	*permission = std::move(r.permission);
+	if (permission != nullptr)
+		*permission = std::move(r.permission);
 	return TRUE;
 }
 
@@ -167,8 +178,10 @@ BOOL exmdb_client_remote::get_folder_by_class(const char *dir, const char *str_c
 	q.str_class = deconst(str_class);
 	if (!exmdb_client_do_rpc(&q, &r))
 		return false;
-	*id = std::move(r.id);
-	*str_explicit = std::move(r.str_explicit);
+	if (id != nullptr)
+		*id = std::move(r.id);
+	if (str_explicit != nullptr)
+		*str_explicit = std::move(r.str_explicit);
 	return TRUE;
 }
 
@@ -183,7 +196,8 @@ BOOL exmdb_client_remote::set_folder_by_class(const char *dir, uint64_t folder_i
 	q.str_class = deconst(str_class);
 	if (!exmdb_client_do_rpc(&q, &r))
 		return false;
-	*b_result = std::move(r.b_result);
+	if (b_result != nullptr)
+		*b_result = std::move(r.b_result);
 	return TRUE;
 }
 
@@ -196,7 +210,8 @@ BOOL exmdb_client_remote::get_folder_class_table(const char *dir, TARRAY_SET *ta
 	q.dir = deconst(dir);
 	if (!exmdb_client_do_rpc(&q, &r))
 		return false;
-	*table = std::move(r.table);
+	if (table != nullptr)
+		*table = std::move(r.table);
 	return TRUE;
 }
 
@@ -210,7 +225,8 @@ BOOL exmdb_client_remote::is_folder_present(const char *dir, uint64_t folder_id,
 	q.folder_id = folder_id;
 	if (!exmdb_client_do_rpc(&q, &r))
 		return false;
-	*b_exist = std::move(r.b_exist);
+	if (b_exist != nullptr)
+		*b_exist = std::move(r.b_exist);
 	return TRUE;
 }
 
@@ -224,7 +240,8 @@ BOOL exmdb_client_remote::is_folder_deleted(const char *dir, uint64_t folder_id,
 	q.folder_id = folder_id;
 	if (!exmdb_client_do_rpc(&q, &r))
 		return false;
-	*b_del = std::move(r.b_del);
+	if (b_del != nullptr)
+		*b_del = std::move(r.b_del);
 	return TRUE;
 }
 
@@ -239,7 +256,8 @@ BOOL exmdb_client_remote::get_folder_by_name(const char *dir, uint64_t parent_id
 	q.str_name = deconst(str_name);
 	if (!exmdb_client_do_rpc(&q, &r))
 		return false;
-	*folder_id = std::move(r.folder_id);
+	if (folder_id != nullptr)
+		*folder_id = std::move(r.folder_id);
 	return TRUE;
 }
 
@@ -254,7 +272,8 @@ BOOL exmdb_client_remote::get_folder_perm(const char *dir, uint64_t folder_id, c
 	q.username = deconst(username);
 	if (!exmdb_client_do_rpc(&q, &r))
 		return false;
-	*permission = std::move(r.permission);
+	if (permission != nullptr)
+		*permission = std::move(r.permission);
 	return TRUE;
 }
 
@@ -269,7 +288,8 @@ BOOL exmdb_client_remote::create_folder_v1(const char *dir, cpid_t cpid, const T
 	q.pproperties = deconst(pproperties);
 	if (!exmdb_client_do_rpc(&q, &r))
 		return false;
-	*folder_id = std::move(r.folder_id);
+	if (folder_id != nullptr)
+		*folder_id = std::move(r.folder_id);
 	return TRUE;
 }
 
@@ -284,8 +304,10 @@ BOOL exmdb_client_remote::create_folder(const char *dir, cpid_t cpid, const TPRO
 	q.pproperties = deconst(pproperties);
 	if (!exmdb_client_do_rpc(&q, &r))
 		return false;
-	*folder_id = std::move(r.folder_id);
-	*e_result = std::move(r.e_result);
+	if (folder_id != nullptr)
+		*folder_id = std::move(r.folder_id);
+	if (e_result != nullptr)
+		*e_result = std::move(r.e_result);
 	return TRUE;
 }
 
@@ -299,7 +321,8 @@ BOOL exmdb_client_remote::get_folder_all_proptags(const char *dir, uint64_t fold
 	q.folder_id = folder_id;
 	if (!exmdb_client_do_rpc(&q, &r))
 		return false;
-	*proptags = std::move(r.proptags);
+	if (proptags != nullptr)
+		*proptags = std::move(r.proptags);
 	return TRUE;
 }
 
@@ -315,7 +338,8 @@ BOOL exmdb_client_remote::get_folder_properties(const char *dir, cpid_t cpid, ui
 	q.pproptags = deconst(pproptags);
 	if (!exmdb_client_do_rpc(&q, &r))
 		return false;
-	*propvals = std::move(r.propvals);
+	if (propvals != nullptr)
+		*propvals = std::move(r.propvals);
 	return TRUE;
 }
 
@@ -331,7 +355,8 @@ BOOL exmdb_client_remote::set_folder_properties(const char *dir, cpid_t cpid, ui
 	q.pproperties = deconst(pproperties);
 	if (!exmdb_client_do_rpc(&q, &r))
 		return false;
-	*problems = std::move(r.problems);
+	if (problems != nullptr)
+		*problems = std::move(r.problems);
 	return TRUE;
 }
 
@@ -361,7 +386,8 @@ BOOL exmdb_client_remote::delete_folder(const char *dir, cpid_t cpid, uint64_t f
 	q.b_hard = b_hard;
 	if (!exmdb_client_do_rpc(&q, &r))
 		return false;
-	*b_result = std::move(r.b_result);
+	if (b_result != nullptr)
+		*b_result = std::move(r.b_result);
 	return TRUE;
 }
 
@@ -378,7 +404,8 @@ BOOL exmdb_client_remote::empty_folder(const char *dir, cpid_t cpid, const char
 	q.flags = flags;
 	if (!exmdb_client_do_rpc(&q, &r))
 		return false;
-	*b_partial = std::move(r.b_partial);
+	if (b_partial != nullptr)
+		*b_partial = std::move(r.b_partial);
 	return TRUE;
 }
 
@@ -393,7 +420,8 @@ BOOL exmdb_client_remote::is_descendant_folder(const char *dir, uint64_t parent_
 	q.child_fid = child_fid;
 	if (!exmdb_client_do_rpc(&q, &r))
 		return false;
-	*b_included = std::move(r.b_included);
+	if (b_included != nullptr)
+		*b_included = std::move(r.b_included);
 	return TRUE;
 }
 
@@ -414,8 +442,10 @@ BOOL exmdb_client_remote::copy_folder_internal(const char *dir, cpid_t cpid, BOO
 	q.dst_fid = dst_fid;
 	if (!exmdb_client_do_rpc(&q, &r))
 		return false;
-	*b_collid = std::move(r.b_collid);
-	*b_partial = std::move(r.b_partial);
+	if (b_collid != nullptr)
+		*b_collid = std::move(r.b_collid);
+	if (b_partial != nullptr)
+		*b_partial = std::move(r.b_partial);
 	return TRUE;
 }
 
@@ -429,9 +459,12 @@ BOOL exmdb_client_remote::get_search_criteria(const char *dir, uint64_t folder_i
 	q.folder_id = folder_id;
 	if (!exmdb_client_do_rpc(&q, &r))
 		return false;
-	*search_status = std::move(r.search_status);
-	*prestriction = std::move(r.prestriction);
-	*folder_ids = std::move(r.folder_ids);
+	if (search_status != nullptr)
+		*search_status = std::move(r.search_status);
+	if (prestriction != nullptr)
+		*prestriction = std::move(r.prestriction);
+	if (folder_ids != nullptr)
+		*folder_ids = std::move(r.folder_ids);
 	return TRUE;
 }
 
@@ -449,7 +482,8 @@ BOOL exmdb_client_remote::set_search_criteria(const char *dir, cpid_t cpid, uint
 	q.pfolder_ids = deconst(pfolder_ids);
 	if (!exmdb_client_do_rpc(&q, &r))
 		return false;
-	*b_result = std::move(r.b_result);
+	if (b_result != nullptr)
+		*b_result = std::move(r.b_result);
 	return TRUE;
 }
 
@@ -467,7 +501,8 @@ BOOL exmdb_client_remote::movecopy_message(const char *dir, cpid_t cpid, uint64_
 	q.b_move = b_move;
 	if (!exmdb_client_do_rpc(&q, &r))
 		return false;
-	*b_result = std::move(r.b_result);
+	if (b_result != nullptr)
+		*b_result = std::move(r.b_result);
 	return TRUE;
 }
 
@@ -487,7 +522,8 @@ BOOL exmdb_client_remote::movecopy_messages(const char *dir, cpid_t cpid, BOOL b
 	q.pmessage_ids = deconst(pmessage_ids);
 	if (!exmdb_client_do_rpc(&q, &r))
 		return false;
-	*b_partial = std::move(r.b_partial);
+	if (b_partial != nullptr)
+		*b_partial = std::move(r.b_partial);
 	return TRUE;
 }
 
@@ -508,7 +544,8 @@ BOOL exmdb_client_remote::movecopy_folder(const char *dir, cpid_t cpid, BOOL b_g
 	q.b_copy = b_copy;
 	if (!exmdb_client_do_rpc(&q, &r))
 		return false;
-	*e_result = std::move(r.e_result);
+	if (e_result != nullptr)
+		*e_result = std::move(r.e_result);
 	return TRUE;
 }
 
@@ -526,7 +563,8 @@ BOOL exmdb_client_remote::delete_messages(const char *dir, cpid_t cpid, const ch
 	q.b_hard = b_hard;
 	if (!exmdb_client_do_rpc(&q, &r))
 		return false;
-	*b_partial = std::move(r.b_partial);
+	if (b_partial != nullptr)
+		*b_partial = std::move(r.b_partial);
 	return TRUE;
 }
 
@@ -541,7 +579,8 @@ BOOL exmdb_client_remote::get_message_brief(const char *dir, cpid_t cpid, uint64
 	q.message_id = message_id;
 	if (!exmdb_client_do_rpc(&q, &r))
 		return false;
-	*pbrief = std::move(r.pbrief);
+	if (pbrief != nullptr)
+		*pbrief = std::move(r.pbrief);
 	return TRUE;
 }
 
@@ -557,7 +596,8 @@ BOOL exmdb_client_remote::sum_hierarchy(const char *dir, uint64_t folder_id, con
 	q.b_depth = b_depth;
 	if (!exmdb_client_do_rpc(&q, &r))
 		return false;
-	*count = std::move(r.count);
+	if (count != nullptr)
+		*count = std::move(r.count);
 	return TRUE;
 }
 
@@ -574,8 +614,10 @@ BOOL exmdb_client_remote::load_hierarchy_table(const char *dir, uint64_t folder_
 	q.prestriction = deconst(prestriction);
 	if (!exmdb_client_do_rpc(&q, &r))
 		return false;
-	*table_id = std::move(r.table_id);
-	*row_count = std::move(r.row_count);
+	if (table_id != nullptr)
+		*table_id = std::move(r.table_id);
+	if (row_count != nullptr)
+		*row_count = std::move(r.row_count);
 	return TRUE;
 }
 
@@ -591,7 +633,8 @@ BOOL exmdb_client_remote::sum_content(const char *dir, uint64_t folder_id, BOOL
 	q.b_deleted = b_deleted;
 	if (!exmdb_client_do_rpc(&q, &r))
 		return false;
-	*count = std::move(r.count);
+	if (count != nullptr)
+		*count = std::move(r.count);
 	return TRUE;
 }
 
@@ -610,8 +653,10 @@ BOOL exmdb_client_remote::load_content_table(const char *dir, cpid_t cpid, uint6
 	q.psorts = deconst(psorts);
 	if (!exmdb_client_do_rpc(&q, &r))
 		return false;
-	*table_id = std::move(r.table_id);
-	*row_count = std::move(r.row_count);
+	if (table_id != nullptr)
+		*table_id = std::move(r.table_id);
+	if (row_count != nullptr)
+		*row_count = std::move(r.row_count);
 	return TRUE;
 }
 
@@ -639,8 +684,10 @@ BOOL exmdb_client_remote::load_permission_table(const char *dir, uint64_t folder
 	q.table_flags = table_flags;
 	if (!exmdb_client_do_rpc(&q, &r))
 		return false;
-	*table_id = std::move(r.table_id);
-	*row_count = std::move(r.row_count);
+	if (table_id != nullptr)
+		*table_id = std::move(r.table_id);
+	if (row_count != nullptr)
+		*row_count = std::move(r.row_count);
 	return TRUE;
 }
 
@@ -656,8 +703,10 @@ BOOL exmdb_client_remote::load_rule_table(const char *dir, uint64_t folder_id, u
 	q.prestriction = deconst(prestriction);
 	if (!exmdb_client_do_rpc(&q, &r))
 		return false;
-	*table_id = std::move(r.table_id);
-	*row_count = std::move(r.row_count);
+	if (table_id != nullptr)
+		*table_id = std::move(r.table_id);
+	if (row_count != nullptr)
+		*row_count = std::move(r.row_count);
 	return TRUE;
 }
 
@@ -684,7 +733,8 @@ BOOL exmdb_client_remote::sum_table(const char *dir, uint32_t table_id, uint32_t
 	q.table_id = table_id;
 	if (!exmdb_client_do_rpc(&q, &r))
 		return false;
-	*rows = std::move(r.rows);
+	if (rows != nullptr)
+		*rows = std::move(r.rows);
 	return TRUE;
 }
 
@@ -703,7 +753,8 @@ BOOL exmdb_client_remote::query_table(const char *dir, const char *username, cpi
 	q.row_needed = row_needed;
 	if (!exmdb_client_do_rpc(&q, &r))
 		return false;
-	*set = std::move(r.set);
+	if (set != nullptr)
+		*set = std::move(r.set);
 	return TRUE;
 }
 
@@ -723,8 +774,10 @@ BOOL exmdb_client_remote::match_table(const char *dir, const char *username, cpi
 	q.pproptags = deconst(pproptags);
 	if (!exmdb_client_do_rpc(&q, &r))
 		return false;
-	*position = std::move(r.position);
-	*propvals = std::move(r.propvals);
+	if (position != nullptr)
+		*position = std::move(r.position);
+	if (propvals != nullptr)
+		*propvals = std::move(r.propvals);
 	return TRUE;
 }
 
@@ -740,8 +793,10 @@ BOOL exmdb_client_remote::locate_table(const char *dir, uint32_t table_id, uint6
 	q.inst_num = inst_num;
 	if (!exmdb_client_do_rpc(&q, &r))
 		return false;
-	*position = std::move(r.position);
-	*row_type = std::move(r.row_type);
+	if (position != nullptr)
+		*position = std::move(r.position);
+	if (row_type != nullptr)
+		*row_type = std::move(r.row_type);
 	return TRUE;
 }
 
@@ -760,7 +815,8 @@ BOOL exmdb_client_remote::read_table_row(const char *dir, const char *username,
 	q.inst_num = inst_num;
 	if (!exmdb_client_do_rpc(&q, &r))
 		return false;
-	*propvals = std::move(r.propvals);
+	if (propvals != nullptr)
+		*propvals = std::move(r.propvals);
 	return TRUE;
 }
 
@@ -775,9 +831,12 @@ BOOL exmdb_client_remote::mark_table(const char *dir, uint32_t table_id, uint32_
 	q.position = position;
 	if (!exmdb_client_do_rpc(&q, &r))
 		return false;
-	*inst_id = std::move(r.inst_id);
-	*inst_num = std::move(r.inst_num);
-	*row_type = std::move(r.row_type);
+	if (inst_id != nullptr)
+		*inst_id = std::move(r.inst_id);
+	if (inst_num != nullptr)
+		*inst_num = std::move(r.inst_num);
+	if (row_type != nullptr)
+		*row_type = std::move(r.row_type);
 	return TRUE;
 }
 
@@ -791,7 +850,8 @@ BOOL exmdb_client_remote::get_table_all_proptags(const char *dir, uint32_t table
 	q.table_id = table_id;
 	if (!exmdb_client_do_rpc(&q, &r))
 		return false;
-	*proptags = std::move(r.proptags);
+	if (proptags != nullptr)
+		*proptags = std::move(r.proptags);
 	return TRUE;
 }
 
@@ -806,9 +866,12 @@ BOOL exmdb_client_remote::expand_table(const char *dir, uint32_t table_id, uint6
 	q.inst_id = inst_id;
 	if (!exmdb_client_do_rpc(&q, &r))
 		return false;
-	*b_found = std::move(r.b_found);
-	*position = std::move(r.position);
-	*row_count = std::move(r.row_count);
+	if (b_found != nullptr)
+		*b_found = std::move(r.b_found);
+	if (position != nullptr)
+		*position = std::move(r.position);
+	if (row_count != nullptr)
+		*row_count = std::move(r.row_count);
 	return TRUE;
 }
 
@@ -823,9 +886,12 @@ BOOL exmdb_client_remote::collapse_table(const char *dir, uint32_t table_id, uin
 	q.inst_id = inst_id;
 	if (!exmdb_client_do_rpc(&q, &r))
 		return false;
-	*b_found = std::move(r.b_found);
-	*position = std::move(r.position);
-	*row_count = std::move(r.row_count);
+	if (b_found != nullptr)
+		*b_found = std::move(r.b_found);
+	if (position != nullptr)
+		*position = std::move(r.position);
+	if (row_count != nullptr)
+		*row_count = std::move(r.row_count);
 	return TRUE;
 }
 
@@ -841,7 +907,8 @@ BOOL exmdb_client_remote::store_table_state(const char *dir, uint32_t table_id,
 	q.inst_num = inst_num;
 	if (!exmdb_client_do_rpc(&q, &r))
 		return false;
-	*state_id = std::move(r.state_id);
+	if (state_id != nullptr)
+		*state_id = std::move(r.state_id);
 	return TRUE;
 }
 
@@ -856,7 +923,8 @@ BOOL exmdb_client_remote::restore_table_state(const char *dir, uint32_t table_id
 	q.state_id = state_id;
 	if (!exmdb_client_do_rpc(&q, &r))
 		return false;
-	*position = std::move(r.position);
+	if (position != nullptr)
+		*position = std::move(r.position);
 	return TRUE;
 }
 
@@ -871,7 +939,8 @@ BOOL exmdb_client_remote::is_msg_present(const char *dir, uint64_t folder_id, ui
 	q.message_id = message_id;
 	if (!exmdb_client_do_rpc(&q, &r))
 		return false;
-	*b_exist = std::move(r.b_exist);
+	if (b_exist != nullptr)
+		*b_exist = std::move(r.b_exist);
 	return TRUE;
 }
 
@@ -885,7 +954,8 @@ BOOL exmdb_client_remote::is_msg_deleted(const char *dir, uint64_t message_id, B
 	q.message_id = message_id;
 	if (!exmdb_client_do_rpc(&q, &r))
 		return false;
-	*b_del = std::move(r.b_del);
+	if (b_del != nullptr)
+		*b_del = std::move(r.b_del);
 	return TRUE;
 }
 
@@ -903,7 +973,8 @@ BOOL exmdb_client_remote::load_message_instance(const char *dir, const char *use
 	q.message_id = message_id;
 	if (!exmdb_client_do_rpc(&q, &r))
 		return false;
-	*instance_id = std::move(r.instance_id);
+	if (instance_id != nullptr)
+		*instance_id = std::move(r.instance_id);
 	return TRUE;
 }
 
@@ -918,7 +989,8 @@ BOOL exmdb_client_remote::load_embedded_instance(const char *dir, BOOL b_new, ui
 	q.attachment_instance_id = attachment_instance_id;
 	if (!exmdb_client_do_rpc(&q, &r))
 		return false;
-	*instance_id = std::move(r.instance_id);
+	if (instance_id != nullptr)
+		*instance_id = std::move(r.instance_id);
 	return TRUE;
 }
 
@@ -932,7 +1004,8 @@ BOOL exmdb_client_remote::get_embedded_cn(const char *dir, uint32_t instance_id,
 	q.instance_id = instance_id;
 	if (!exmdb_client_do_rpc(&q, &r))
 		return false;
-	*pcn = std::move(r.pcn);
+	if (pcn != nullptr)
+		*pcn = std::move(r.pcn);
 	return TRUE;
 }
 
@@ -946,7 +1019,8 @@ BOOL exmdb_client_remote::reload_message_instance(const char *dir, uint32_t inst
 	q.instance_id = instance_id;
 	if (!exmdb_client_do_rpc(&q, &r))
 		return false;
-	*b_result = std::move(r.b_result);
+	if (b_result != nullptr)
+		*b_result = std::move(r.b_result);
 	return TRUE;
 }
 
@@ -973,7 +1047,8 @@ BOOL exmdb_client_remote::read_message_instance(const char *dir, uint32_t instan
 	q.instance_id = instance_id;
 	if (!exmdb_client_do_rpc(&q, &r))
 		return false;
-	*msgctnt = std::move(r.msgctnt);
+	if (msgctnt != nullptr)
+		*msgctnt = std::move(r.msgctnt);
 	return TRUE;
 }
 
@@ -989,8 +1064,10 @@ BOOL exmdb_client_remote::write_message_instance(const char *dir, uint32_t insta
 	q.b_force = b_force;
 	if (!exmdb_client_do_rpc(&q, &r))
 		return false;
-	*proptags = std::move(r.proptags);
-	*problems = std::move(r.problems);
+	if (proptags != nullptr)
+		*proptags = std::move(r.proptags);
+	if (problems != nullptr)
+		*problems = std::move(r.problems);
 	return TRUE;
 }
 
@@ -1005,7 +1082,8 @@ BOOL exmdb_client_remote::load_attachment_instance(const char *dir, uint32_t mes
 	q.attachment_num = attachment_num;
 	if (!exmdb_client_do_rpc(&q, &r))
 		return false;
-	*instance_id = std::move(r.instance_id);
+	if (instance_id != nullptr)
+		*instance_id = std::move(r.instance_id);
 	return TRUE;
 }
 
@@ -1019,8 +1097,10 @@ BOOL exmdb_client_remote::create_attachment_instance(const char *dir, uint32_t m
 	q.message_instance_id = message_instance_id;
 	if (!exmdb_client_do_rpc(&q, &r))
 		return false;
-	*instance_id = std::move(r.instance_id);
-	*attachment_num = std::move(r.attachment_num);
+	if (instance_id != nullptr)
+		*instance_id = std::move(r.instance_id);
+	if (attachment_num != nullptr)
+		*attachment_num = std::move(r.attachment_num);
 	return TRUE;
 }
 
@@ -1034,7 +1114,8 @@ BOOL exmdb_client_remote::read_attachment_instance(const char *dir, uint32_t ins
 	q.instance_id = instance_id;
 	if (!exmdb_client_do_rpc(&q, &r))
 		return false;
-	*attctnt = std::move(r.attctnt);
+	if (attctnt != nullptr)
+		*attctnt = std::move(r.attctnt);
 	return TRUE;
 }
 
@@ -1050,7 +1131,8 @@ BOOL exmdb_client_remote::write_attachment_instance(const char *dir, uint32_t in
 	q.b_force = b_force;
 	if (!exmdb_client_do_rpc(&q, &r))
 		return false;
-	*problems = std::move(r.problems);
+	if (problems != nullptr)
+		*problems = std::move(r.problems);
 	return TRUE;
 }
 
@@ -1078,7 +1160,8 @@ BOOL exmdb_client_remote::flush_instance(const char *dir, uint32_t instance_id,
 	q.instance_id = instance_id;
 	if (!exmdb_client_do_rpc(&q, &r))
 		return false;
-	*e_result = std::move(r.e_result);
+	if (e_result != nullptr)
+		*e_result = std::move(r.e_result);
 	return TRUE;
 }
 
@@ -1105,7 +1188,8 @@ BOOL exmdb_client_remote::get_instance_all_proptags(const char *dir, uint32_t in
 	q.instance_id = instance_id;
 	if (!exmdb_client_do_rpc(&q, &r))
 		return false;
-	*proptags = std::move(r.proptags);
+	if (proptags != nullptr)
+		*proptags = std::move(r.proptags);
 	return TRUE;
 }
 
@@ -1121,7 +1205,8 @@ BOOL exmdb_client_remote::get_instance_properties(const char *dir, uint32_t size
 	q.pproptags = deconst(pproptags);
 	if (!exmdb_client_do_rpc(&q, &r))
 		return false;
-	*propvals = std::move(r.propvals);
+	if (propvals != nullptr)
+		*propvals = std::move(r.propvals);
 	return TRUE;
 }
 
@@ -1136,7 +1221,8 @@ BOOL exmdb_client_remote::set_instance_properties(const char *dir, uint32_t inst
 	q.pproperties = deconst(pproperties);
 	if (!exmdb_client_do_rpc(&q, &r))
 		return false;
-	*problems = std::move(r.problems);
+	if (problems != nullptr)
+		*problems = std::move(r.problems);
 	return TRUE;
 }
 
@@ -1151,7 +1237,8 @@ BOOL exmdb_client_remote::remove_instance_properties(const char *dir, uint32_t i
 	q.pproptags = deconst(pproptags);
 	if (!exmdb_client_do_rpc(&q, &r))
 		return false;
-	*problems = std::move(r.problems);
+	if (problems != nullptr)
+		*problems = std::move(r.problems);
 	return TRUE;
 }
 
@@ -1166,7 +1253,8 @@ BOOL exmdb_client_remote::is_descendant_instance(const char *dir, uint32_t paren
 	q.child_iid = child_iid;
 	if (!exmdb_client_do_rpc(&q, &r))
 		return false;
-	*b_included = std::move(r.b_included);
+	if (b_included != nullptr)
+		*b_included = std::move(r.b_included);
 	return TRUE;
 }
 
@@ -1193,7 +1281,8 @@ BOOL exmdb_client_remote::get_message_instance_rcpts_num(const char *dir, uint32
 	q.instance_id = instance_id;
 	if (!exmdb_client_do_rpc(&q, &r))
 		return false;
-	*num = std::move(r.num);
+	if (num != nullptr)
+		*num = std::move(r.num);
 	return TRUE;
 }
 
@@ -1207,7 +1296,8 @@ BOOL exmdb_client_remote::get_message_instance_rcpts_all_proptags(const char *di
 	q.instance_id = instance_id;
 	if (!exmdb_client_do_rpc(&q, &r))
 		return false;
-	*proptags = std::move(r.proptags);
+	if (proptags != nullptr)
+		*proptags = std::move(r.proptags);
 	return TRUE;
 }
 
@@ -1223,7 +1313,8 @@ BOOL exmdb_client_remote::get_message_instance_rcpts(const char *dir, uint32_t i
 	q.need_count = need_count;
 	if (!exmdb_client_do_rpc(&q, &r))
 		return false;
-	*set = std::move(r.set);
+	if (set != nullptr)
+		*set = std::move(r.set);
 	return TRUE;
 }
 
@@ -1253,7 +1344,8 @@ BOOL exmdb_client_remote::copy_instance_rcpts(const char *dir, BOOL b_force, uin
 	q.dst_instance_id = dst_instance_id;
 	if (!exmdb_client_do_rpc(&q, &r))
 		return false;
-	*b_result = std::move(r.b_result);
+	if (b_result != nullptr)
+		*b_result = std::move(r.b_result);
 	return TRUE;
 }
 
@@ -1280,7 +1372,8 @@ BOOL exmdb_client_remote::get_message_instance_attachments_num(const char *dir,
 	q.instance_id = instance_id;
 	if (!exmdb_client_do_rpc(&q, &r))
 		return false;
-	*num = std::move(r.num);
+	if (num != nullptr)
+		*num = std::move(r.num);
 	return TRUE;
 }
 
@@ -1294,7 +1387,8 @@ BOOL exmdb_client_remote::get_message_instance_attachment_table_all_proptags(con
 	q.instance_id = instance_id;
 	if (!exmdb_client_do_rpc(&q, &r))
 		return false;
-	*proptags = std::move(r.proptags);
+	if (proptags != nullptr)
+		*proptags = std::move(r.proptags);
 	return TRUE;
 }
 
@@ -1311,7 +1405,8 @@ BOOL exmdb_client_remote::query_message_instance_attachment_table(const char *di
 	q.row_needed = row_needed;
 	if (!exmdb_client_do_rpc(&q, &r))
 		return false;
-	*set = std::move(r.set);
+	if (set != nullptr)
+		*set = std::move(r.set);
 	return TRUE;
 }
 
@@ -1327,7 +1422,8 @@ BOOL exmdb_client_remote::copy_instance_attachments(const char *dir, BOOL b_forc
 	q.dst_instance_id = dst_instance_id;
 	if (!exmdb_client_do_rpc(&q, &r))
 		return false;
-	*b_result = std::move(r.b_result);
+	if (b_result != nullptr)
+		*b_result = std::move(r.b_result);
 	return TRUE;
 }
 
@@ -1355,7 +1451,8 @@ BOOL exmdb_client_remote::get_message_rcpts(const char *dir, uint64_t message_id
 	q.message_id = message_id;
 	if (!exmdb_client_do_rpc(&q, &r))
 		return false;
-	*set = std::move(r.set);
+	if (set != nullptr)
+		*set = std::move(r.set);
 	return TRUE;
 }
 
@@ -1372,7 +1469,8 @@ BOOL exmdb_client_remote::get_message_properties(const char *dir, const char *us
 	q.pproptags = deconst(pproptags);
 	if (!exmdb_client_do_rpc(&q, &r))
 		return false;
-	*propvals = std::move(r.propvals);
+	if (propvals != nullptr)
+		*propvals = std::move(r.propvals);
 	return TRUE;
 }
 
@@ -1389,7 +1487,8 @@ BOOL exmdb_client_remote::set_message_properties(const char *dir, const char *us
 	q.pproperties = deconst(pproperties);
 	if (!exmdb_client_do_rpc(&q, &r))
 		return false;
-	*problems = std::move(r.problems);
+	if (problems != nullptr)
+		*problems = std::move(r.problems);
 	return TRUE;
 }
 
@@ -1405,7 +1504,8 @@ BOOL exmdb_client_remote::set_message_read_state(const char *dir, const char *us
 	q.mark_as_read = mark_as_read;
 	if (!exmdb_client_do_rpc(&q, &r))
 		return false;
-	*read_cn = std::move(r.read_cn);
+	if (read_cn != nullptr)
+		*read_cn = std::move(r.read_cn);
 	return TRUE;
 }
 
@@ -1434,7 +1534,8 @@ BOOL exmdb_client_remote::allocate_message_id(const char *dir, uint64_t folder_i
 	q.folder_id = folder_id;
 	if (!exmdb_client_do_rpc(&q, &r))
 		return false;
-	*message_id = std::move(r.message_id);
+	if (message_id != nullptr)
+		*message_id = std::move(r.message_id);
 	return TRUE;
 }
 
@@ -1447,7 +1548,8 @@ BOOL exmdb_client_remote::allocate_cn(const char *dir, uint64_t *cn)
 	q.dir = deconst(dir);
 	if (!exmdb_client_do_rpc(&q, &r))
 		return false;
-	*cn = std::move(r.cn);
+	if (cn != nullptr)
+		*cn = std::move(r.cn);
 	return TRUE;
 }
 
@@ -1474,7 +1576,8 @@ BOOL exmdb_client_remote::try_mark_submit(const char *dir, uint64_t message_id,
 	q.message_id = message_id;
 	if (!exmdb_client_do_rpc(&q, &r))
 		return false;
-	*b_marked = std::move(r.b_marked);
+	if (b_marked != nullptr)
+		*b_marked = std::move(r.b_marked);
 	return TRUE;
 }
 
@@ -1504,7 +1607,8 @@ BOOL exmdb_client_remote::link_message(const char *dir, cpid_t cpid, uint64_t fo
 	q.message_id = message_id;
 	if (!exmdb_client_do_rpc(&q, &r))
 		return false;
-	*b_result = std::move(r.b_result);
+	if (b_result != nullptr)
+		*b_result = std::move(r.b_result);
 	return TRUE;
 }
 
@@ -1563,7 +1667,8 @@ BOOL exmdb_client_remote::get_message_timer(const char *dir, uint64_t message_id
 	q.message_id = message_id;
 	if (!exmdb_client_do_rpc(&q, &r))
 		return false;
-	*ptimer_id = std::move(r.ptimer_id);
+	if (ptimer_id != nullptr)
+		*ptimer_id = std::move(r.ptimer_id);
 	return TRUE;
 }
 
@@ -1621,7 +1726,8 @@ BOOL exmdb_client_remote::update_folder_rule(const char *dir, uint64_t folder_id
 	q.prow = deconst(prow);
 	if (!exmdb_client_do_rpc(&q, &r))
 		return false;
-	*b_exceed = std::move(r.b_exceed);
+	if (b_exceed != nullptr)
+		*b_exceed = std::move(r.b_exceed);
 	return TRUE;
 }
 
@@ -1640,9 +1746,12 @@ BOOL exmdb_client_remote::deliver_message(const char *dir, const char *from_addr
 	q.pdigest = deconst(pdigest);
 	if (!exmdb_client_do_rpc(&q, &r))
 		return false;
-	*folder_id = std::move(r.folder_id);
-	*message_id = std::move(r.message_id);
-	*result = std::move(r.result);
+	if (folder_id != nullptr)
+		*folder_id = std::move(r.folder_id);
+	if (message_id != nullptr)
+		*message_id = std::move(r.message_id);
+	if (result != nullptr)
+		*result = std::move(r.result);
 	return TRUE;
 }
 
@@ -1659,9 +1768,12 @@ BOOL exmdb_client_remote::write_message(const char *dir, cpid_t cpid, uint64_t f
 	q.digest = digest;
 	if (!exmdb_client_do_rpc(&q, &r))
 		return false;
-	*outmid = std::move(r.outmid);
-	*outcn = std::move(r.outcn);
-	*e_result = std::move(r.e_result);
+	if (outmid != nullptr)
+		*outmid = std::move(r.outmid);
+	if (outcn != nullptr)
+		*outcn = std::move(r.outcn);
+	if (e_result != nullptr)
+		*e_result = std::move(r.e_result);
 	return TRUE;
 }
 
@@ -1677,11 +1789,12 @@ BOOL exmdb_client_remote::read_message(const char *dir, const char *username, cp
 	q.message_id = message_id;
 	if (!exmdb_client_do_rpc(&q, &r))
 		return false;
-	*pmsgctnt = std::move(r.pmsgctnt);
+	if (pmsgctnt != nullptr)
+		*pmsgctnt = std::move(r.pmsgctnt);
 	return TRUE;
 }
 
-BOOL exmdb_client_remote::get_content_sync(const char *dir, uint64_t folder_id, const char *username, const idset *pgiven, const idset *pseen, const idset *pseen_fai, const idset *pread, cpid_t cpid, const RESTRICTION *prestriction, BOOL b_ordered, uint32_t *fai_count, uint64_t *fai_total, uint32_t *normal_count, uint64_t *normal_total, EID_ARRAY *updated_mids, EID_ARRAY *chg_mids, uint64_t *last_cn, EID_ARRAY *given_mids, EID_ARRAY *deleted_mids, EID_ARRAY *nolonger_mids, EID_ARRAY *read_mids, EID_ARRAY *unread_mids, uint64_t *last_readcn)
+BOOL exmdb_client_remote::get_content_sync(const char *dir, uint64_t folder_id, const char *username, const gromox::idset *pgiven, const gromox::idset *pseen, const gromox::idset *pseen_fai, const gromox::idset *pread, cpid_t cpid, const RESTRICTION *prestriction, BOOL b_ordered, uint32_t *fai_count, uint64_t *fai_total, uint32_t *normal_count, uint64_t *normal_total, EID_ARRAY *updated_mids, EID_ARRAY *chg_mids, uint64_t *last_cn, EID_ARRAY *given_mids, EID_ARRAY *deleted_mids, EID_ARRAY *nolonger_mids, EID_ARRAY *read_mids, EID_ARRAY *unread_mids, uint64_t *last_readcn)
 {
 	exreq_get_content_sync q{};
 	exresp_get_content_sync r{};
@@ -1699,23 +1812,36 @@ BOOL exmdb_client_remote::get_content_sync(const char *dir, uint64_t folder_id,
 	q.b_ordered = b_ordered;
 	if (!exmdb_client_do_rpc(&q, &r))
 		return false;
-	*fai_count = std::move(r.fai_count);
-	*fai_total = std::move(r.fai_total);
-	*normal_count = std::move(r.normal_count);
-	*normal_total = std::move(r.normal_total);
-	*updated_mids = std::move(r.updated_mids);
-	*chg_mids = std::move(r.chg_mids);
-	*last_cn = std::move(r.last_cn);
-	*given_mids = std::move(r.given_mids);
-	*deleted_mids = std::move(r.deleted_mids);
-	*nolonger_mids = std::move(r.nolonger_mids);
-	*read_mids = std::move(r.read_mids);
-	*unread_mids = std::move(r.unread_mids);
-	*last_readcn = std::move(r.last_readcn);
+	if (fai_count != nullptr)
+		*fai_count = std::move(r.fai_count);
+	if (fai_total != nullptr)
+		*fai_total = std::move(r.fai_total);
+	if (normal_count != nullptr)
+		*normal_count = std::move(r.normal_count);
+	if (normal_total != nullptr)
+		*normal_total = std::move(r.normal_total);
+	if (updated_mids != nullptr)
+		*updated_mids = std::move(r.updated_mids);
+	if (chg_mids != nullptr)
+		*chg_mids = std::move(r.chg_mids);
+	if (last_cn != nullptr)
+		*last_cn = std::move(r.last_cn);
+	if (given_mids != nullptr)
+		*given_mids = std::move(r.given_mids);
+	if (deleted_mids != nullptr)
+		*deleted_mids = std::move(r.deleted_mids);
+	if (nolonger_mids != nullptr)
+		*nolonger_mids = std::move(r.nolonger_mids);
+	if (read_mids != nullptr)
+		*read_mids = std::move(r.read_mids);
+	if (unread_mids != nullptr)
+		*unread_mids = std::move(r.unread_mids);
+	if (last_readcn != nullptr)
+		*last_readcn = std::move(r.last_readcn);
 	return TRUE;
 }
 
-BOOL exmdb_client_remote::get_hierarchy_sync(const char *dir, uint64_t folder_id, const char *username, const idset *pgiven, const idset *pseen, FOLDER_CHANGES *fldchgs, uint64_t *last_cn, EID_ARRAY *given_fids, EID_ARRAY *deleted_fids)
+BOOL exmdb_client_remote::get_hierarchy_sync(const char *dir, uint64_t folder_id, const char *username, const gromox::idset *pgiven, const gromox::idset *pseen, FOLDER_CHANGES *fldchgs, uint64_t *last_cn, EID_ARRAY *given_fids, EID_ARRAY *deleted_fids)
 {
 	exreq_get_hierarchy_sync q{};
 	exresp_get_hierarchy_sync r{};
@@ -1728,10 +1854,14 @@ BOOL exmdb_client_remote::get_hierarchy_sync(const char *dir, uint64_t folder_id
 	q.pseen = deconst(pseen);
 	if (!exmdb_client_do_rpc(&q, &r))
 		return false;
-	*fldchgs = std::move(r.fldchgs);
-	*last_cn = std::move(r.last_cn);
-	*given_fids = std::move(r.given_fids);
-	*deleted_fids = std::move(r.deleted_fids);
+	if (fldchgs != nullptr)
+		*fldchgs = std::move(r.fldchgs);
+	if (last_cn != nullptr)
+		*last_cn = std::move(r.last_cn);
+	if (given_fids != nullptr)
+		*given_fids = std::move(r.given_fids);
+	if (deleted_fids != nullptr)
+		*deleted_fids = std::move(r.deleted_fids);
 	return TRUE;
 }
 
@@ -1745,7 +1875,8 @@ BOOL exmdb_client_remote::allocate_ids(const char *dir, uint32_t count, uint64_t
 	q.count = count;
 	if (!exmdb_client_do_rpc(&q, &r))
 		return false;
-	*begin_eid = std::move(r.begin_eid);
+	if (begin_eid != nullptr)
+		*begin_eid = std::move(r.begin_eid);
 	return TRUE;
 }
 
@@ -1762,7 +1893,8 @@ BOOL exmdb_client_remote::subscribe_notification(const char *dir, uint16_t notif
 	q.message_id = message_id;
 	if (!exmdb_client_do_rpc(&q, &r))
 		return false;
-	*sub_id = std::move(r.sub_id);
+	if (sub_id != nullptr)
+		*sub_id = std::move(r.sub_id);
 	return TRUE;
 }
 
@@ -1805,7 +1937,8 @@ BOOL exmdb_client_remote::check_contact_address(const char *dir, const char *pad
 	q.paddress = deconst(paddress);
 	if (!exmdb_client_do_rpc(&q, &r))
 		return false;
-	*b_found = std::move(r.b_found);
+	if (b_found != nullptr)
+		*b_found = std::move(r.b_found);
 	return TRUE;
 }
 
@@ -1820,7 +1953,8 @@ BOOL exmdb_client_remote::get_public_folder_unread_count(const char *dir, const
 	q.folder_id = folder_id;
 	if (!exmdb_client_do_rpc(&q, &r))
 		return false;
-	*count = std::move(r.count);
+	if (count != nullptr)
+		*count = std::move(r.count);
 	return TRUE;
 }
 
@@ -1872,9 +2006,12 @@ BOOL exmdb_client_remote::store_eid_to_user(const char *dir, const STORE_ENTRYID
 	q.store_eid = deconst(store_eid);
 	if (!exmdb_client_do_rpc(&q, &r))
 		return false;
-	*maildir = std::move(r.maildir);
-	*user_id = std::move(r.user_id);
-	*domain_id = std::move(r.domain_id);
+	if (maildir != nullptr)
+		*maildir = std::move(r.maildir);
+	if (user_id != nullptr)
+		*user_id = std::move(r.user_id);
+	if (domain_id != nullptr)
+		*domain_id = std::move(r.domain_id);
 	return TRUE;
 }
 
@@ -1891,10 +2028,14 @@ BOOL exmdb_client_remote::purge_softdelete(const char *dir, const char *username
 	q.cutoff = cutoff;
 	if (!exmdb_client_do_rpc(&q, &r))
 		return false;
-	*cnt_folders = std::move(r.cnt_folders);
-	*cnt_messages = std::move(r.cnt_messages);
-	*sz_normal = std::move(r.sz_normal);
-	*sz_fai = std::move(r.sz_fai);
+	if (cnt_folders != nullptr)
+		*cnt_folders = std::move(r.cnt_folders);
+	if (cnt_messages != nullptr)
+		*cnt_messages = std::move(r.cnt_messages);
+	if (sz_normal != nullptr)
+		*sz_normal = std::move(r.sz_normal);
+	if (sz_fai != nullptr)
+		*sz_fai = std::move(r.sz_fai);
 	return TRUE;
 }
 
@@ -1921,7 +2062,8 @@ BOOL exmdb_client_remote::autoreply_tsquery(const char *dir, const char *peer, u
 	q.window = window;
 	if (!exmdb_client_do_rpc(&q, &r))
 		return false;
-	*tdiff = std::move(r.tdiff);
+	if (tdiff != nullptr)
+		*tdiff = std::move(r.tdiff);
 	return TRUE;
 }
 
@@ -1962,7 +2104,8 @@ BOOL exmdb_client_remote::imapfile_read(const char *dir, const std::string &type
 	q.mid = mid;
 	if (!exmdb_client_do_rpc(&q, &r))
 		return false;
-	*data = std::move(r.data);
+	if (data != nullptr)
+		*data = std::move(r.data);
 	return TRUE;
 }
 
@@ -2032,7 +2175,8 @@ BOOL exmdb_client_remote::autoreply_getprop(const char *dir, cpid_t cpid, const
 	q.pproptags = deconst(pproptags);
 	if (!exmdb_client_do_rpc(&q, &r))
 		return false;
-	*propvals = std::move(r.propvals);
+	if (propvals != nullptr)
+		*propvals = std::move(r.propvals);
 	return TRUE;
 }
 
@@ -2047,7 +2191,8 @@ BOOL exmdb_client_remote::autoreply_setprop(const char *dir, cpid_t cpid, const
 	q.ppropvals = deconst(ppropvals);
 	if (!exmdb_client_do_rpc(&q, &r))
 		return false;
-	*problems = std::move(r.problems);
+	if (problems != nullptr)
+		*problems = std::move(r.problems);
 	return TRUE;
 }
 
